@@ -145,6 +145,11 @@ def Load_SeedCacheFile():
         seed_cache["10Kn"] = f.readlines()
 
 
+# 返回随机的Boolean类型
+def random_boolean(p_arg):
+    return "1" if (random.randint(0, 1) == 1) else "0"
+
+
 # 返回一个随机的时间戳   start < 返回值 < end
 def random_timestamp(p_arg):
     frmt = "%Y-%m-%d %H:%M:%S"
@@ -166,6 +171,15 @@ def random_date(p_arg):
 def random_from_seed(p_arg):
     m_SeedName = str(p_arg[0])
     m_nMaxLength = int(p_arg[1])
+    if m_SeedName.upper().endswith('s'):
+        # 随机字符串，不支持超过100位的字符串
+        if m_nMaxLength > 100:
+            m_nMaxLength = 100
+    if m_SeedName.upper().endswith('n'):
+        # 随机数字，不支持超过20位的数字
+        if m_nMaxLength > 20:
+            m_nMaxLength = 20
+
     if m_SeedName in seed_cache:
         n = len(seed_cache[m_SeedName])
         if n == 0:
@@ -241,7 +255,7 @@ def parse_formula_str(p_formula_str):
     for m_nRowPos in range(0, len(m_row_struct)):
         if re.search('random_ascii_lowercase|random_ascii_uppercase|random_ascii_letters' +
                      '|random_digits|identity|random_ascii_letters_and_digits|random_from_seed' +
-                     '|random_date|random_timestamp',
+                     '|random_date|random_timestamp|random_boolean',
                      m_row_struct[m_nRowPos], re.IGNORECASE):
             m_function_struct = re.split(r'[(,)]', m_row_struct[m_nRowPos])
             for m_nPos in range(0, len(m_function_struct)):
@@ -280,6 +294,9 @@ def parse_formula_str(p_formula_str):
             if m_function_struct[0].upper() == "RANDOM_TIMESTAMP":
                 m_call_out_struct.append(random_timestamp)
                 m_call_out_struct.append(m_function_struct[1:])
+            if m_function_struct[0].upper() == "RANDOM_BOOLEAN":
+                m_call_out_struct.append(random_boolean)
+                m_call_out_struct.append(m_function_struct[1:])
             m_row_struct[m_nRowPos] = m_call_out_struct
     return m_row_struct
 
@@ -308,14 +325,6 @@ def Create_file(p_filename, p_formula_str, p_rows, p_options):
             m_fs = fs.open_fs('./')
             m_filename = p_filename[7:]
             m_output = m_fs.open(m_filename, 'w')
-        elif p_filename.startswith('tar://'):
-            m_fs = fs.open_fs(p_filename + ".tar", create=True)
-            m_filename = p_filename[6:]
-            m_output = m_fs.open(m_filename, 'w')
-        elif p_filename.startswith('zip://'):
-            m_fs = fs.open_fs(p_filename + ".zip",  create=True)
-            m_filename = p_filename[6:]
-            m_output = m_fs.open(m_filename, 'w')
         elif p_filename.startswith('kafka://'):
             if p_options["KAFKA_SERVERS"] is None or len(p_options["KAFKA_SERVERS"].strip() == 0):
                 raise SQLCliException("Please set KAFKA_SERVERS first")
