@@ -11,6 +11,8 @@ class SQLMapping(object):
     # match_roles = [ Key, Value]
     m_SQL_MappingList = {}
 
+    Console = None       # 控制台信息，用来打印日志内容
+
     def Load_SQL_Mappings(self, p_szTestScriptFileName, p_szSQLMappings):
         # 如果不带任何参数，或者参数为空，则清空SQLMapping信息
         if p_szSQLMappings is None:
@@ -33,14 +35,17 @@ class SQLMapping(object):
             m_SQL_MappingBaseName = None
             m_SQL_MappingFullName = None
 
-            # 如果以.map结尾，去掉最后的.map
-            if m_SQL_MappingFile.endswith(".map"):
-                m_SQL_MappingFile = m_SQL_MappingFile[0: len(m_SQL_MappingFile)-4]
-
             if os.path.isfile(m_SQL_MappingFile):
                 # 用户提供的是全路径名
                 m_SQL_MappingBaseName = os.path.basename(m_SQL_MappingFile)  # 不包含路径的文件名
                 m_SQL_MappingFullName = os.path.abspath(m_SQL_MappingFile)
+            elif os.path.isfile(os.path.join(
+                    os.path.dirname(m_szTestScriptFileName),
+                    m_SQL_MappingFile)):
+                # 用户提供的是当前目录下的文件
+                m_SQL_MappingBaseName = os.path.basename(m_SQL_MappingFile)  # 不包含路径的文件名
+                m_SQL_MappingFullName = os.path.join(
+                    os.path.dirname(m_szTestScriptFileName),m_SQL_MappingFile)
             elif os.path.isfile(os.path.join(
                     os.path.dirname(m_szTestScriptFileName),
                     m_SQL_MappingFile + ".map")):
@@ -129,7 +134,12 @@ class SQLMapping(object):
 
     def ReplaceSQL(self, p_szSQL, p_Key, p_Value):
         # 首先查找是否有匹配的内容，如果没有，直接返回
-        m_SearchResult = re.search(p_Key, p_szSQL, re.DOTALL)
+        try:
+            m_SearchResult = re.search(p_Key, p_szSQL, re.DOTALL)
+        except re.error as ex:
+            print("[WARNING] Invalid regex pattern. [" + str(p_Key) + "]  " + repr(ex), file=self.Console)
+            return p_szSQL
+
         if m_SearchResult is None:
             return p_szSQL
         else:
@@ -150,7 +160,12 @@ class SQLMapping(object):
                             m_function_struct[m_nPos] = m_SearchedKey
                     # 执行替换函数
                     if m_function_struct[0].upper() == "ENV":
-                        m_Value = self.ReplaceMacro_Env(m_function_struct[1:])
+                        if len(m_function_struct) <= 1:
+                            print("[WARNING] Invalid env macro, use env(). [" + str(p_Key) + "=>" + str(p_Value) + "]",
+                                  file=self.Console)
+                            m_Value = ""
+                        else:
+                            m_Value = self.ReplaceMacro_Env(m_function_struct[1:])
         m_szSQL = re.sub(p_Key, m_Value, p_szSQL, flags=re.DOTALL)
         return m_szSQL
 
@@ -388,7 +403,8 @@ def SQLAnalyze(p_SQLCommandPlainText):
             strRegexPattern = r'^(\s+)?CREATE(\s+)?|^(\s+)?(\()?SELECT(\s+)?|^(\s+)?UPDATE(\s+)?|' \
                               r'^(\s+)?DELETE(\s+)?|^(\s+)?INSERT(\s+)?|^(\s+)?__INTERNAL__(\s+)?|' \
                               r'^(\s+)?DROP(\s+)?|^(\s+)?REPLACE(\s+)?|^(\s+)?LOAD(\s+)?|' \
-                              r'^(\s+)?MERGE(\s+)?'
+                              r'^(\s+)?MERGE(\s+)?|^(\s+)?DECLARE(\s+)?|^(\s+)?BEGIN(\s+)?|' \
+                              r'^(\s+)?ALTER(\s+)?'
             if not re.match(strRegexPattern, SQLCommands[m_nPos], re.IGNORECASE):
                 SQLSplitResults.append(SQLCommands[m_nPos])
                 SQLSplitResultsWithComments.append(SQLCommandsWithComments[m_nPos])
@@ -424,8 +440,12 @@ def SQLAnalyze(p_SQLCommandPlainText):
             m_NewSQL = SQLCommands[m_nPos]
             m_NewSQLWithComments = SQLCommandsWithComments[m_nPos][m_NewSQLWithCommentsLastPos:]
             if re.match(r'(.*);(\s+)?$', SQLCommands[m_nPos]):  # 本行以；结尾
-                strRegexPattern = r'^((\s+)?CREATE(\s+)|^(\s+)?REPLACE(\s+))(.*)?(FUNCTION|PROCEDURE)'
-                if not re.match(strRegexPattern, SQLCommands[m_nPos], re.IGNORECASE):
+                strRegexPattern = \
+                    r'^((\s+)?CREATE(\s+)|^(\s+)?REPLACE(\s+))(.*)?(FUNCTION|PROCEDURE)'
+                strRegexPattern2 = \
+                    r'^(\s+)?DECLARE(\s+)|^(\s+)?BEGIN(\s+)'
+                if not re.match(strRegexPattern, SQLCommands[m_nPos], re.IGNORECASE) and \
+                        not re.match(strRegexPattern2, SQLCommands[m_nPos], re.IGNORECASE):
                     # 这不是一个存储过程，本行可以结束了
                     SQLSplitResults.append(SQLCommands[m_nPos])
                     SQLSplitResultsWithComments.append(SQLCommandsWithComments[m_nPos])
@@ -513,8 +533,12 @@ def SQLAnalyze(p_SQLCommandPlainText):
                 # 工作在多行语句中，查找;结尾的内容
                 if re.match(r'(.*);(\s+)?$', SQLCommands[m_nPos]):  # 本行以；结尾
                     # 查找这个多行语句是否就是一个存储过程
-                    strRegexPattern = r'(^(\s+)?CREATE|^(\s+)?DROP|^(\s+)?REPLACE)(.*)?\s+(FUNCTION|PROCEDURE)(.*)?'
-                    if not re.match(strRegexPattern, m_NewSQL, re.IGNORECASE):
+                    strRegexPattern = \
+                        r'^((\s+)?CREATE(\s+)|^(\s+)?REPLACE(\s+))(.*)?(FUNCTION|PROCEDURE)'
+                    strRegexPattern2 = \
+                        r'^(\s+)?DECLARE(\s+)|^(\s+)?BEGIN(\s+)'
+                    if not re.match(strRegexPattern, m_NewSQL, re.IGNORECASE) and \
+                            not re.match(strRegexPattern2, m_NewSQL, re.IGNORECASE):
                         # 这不是一个存储过程，遇到了；本行可以结束了
                         SQLSplitResults.append(m_NewSQL)
                         SQLSplitResultsWithComments.append(m_NewSQLWithComments)
@@ -561,7 +585,10 @@ def SQLAnalyze(p_SQLCommandPlainText):
 
     # 去掉SQL语句中的最后一个； ORACLE数据库不支持带有；的语句
     for m_nPos in range(0, len(SQLSplitResults)):
-        if SQLSplitResults[m_nPos][-1:] == ';':
+        # 去掉行尾的空格
+        SQLSplitResults[m_nPos] = SQLSplitResults[m_nPos].rstrip()
+        # 去掉行尾的最后一个分号, “但是如果是END；结尾的，最后的；不能去掉”
+        if SQLSplitResults[m_nPos][-1:] == ';' and not SQLSplitResults[m_nPos].endswith("END;"):
             SQLSplitResults[m_nPos] = SQLSplitResults[m_nPos][:-1]
 
     # 去掉注释信息中的最后一个回车换行符
